// =============================================================================================================
//
//   Topology
//                             P2P                   CSMA   +------ NODE 2   10.1.2.2
//                           10 Mbps                 1Gbps  |
//      NODE 0  SOURCE    <------------>  NODE 1  ----------+------ NODE 3   10.1.2.3
//      10.1.1.1                 10.1.1.2    10.1.2.1       |
//                                                          +------ NODE 4   10.1.2.4
//
//
// This experience download a 6MB file using a simple TCP socket or CCN over udp (depending on user options)
//
// =============================================================================================================
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sstream>
#include "ns3/network-module.h"
#include "ns3/core-module.h"
#include "ns3/internet-module.h"
#include "ns3/dce-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/applications-module.h"
#include "ns3/csma-module.h"
#include "ns3/flow-monitor-helper.h"

#include <algorithm>

using namespace ns3;

void
CreateFiles (std::string ccnTransport, int taille)
{
  FILE *f = fopen("/tmp/bigone","w");
  char buffer[1024];

  memset (buffer, 'A', sizeof (buffer));
  for (int i = 0; i < taille ; i++)
    {
      fwrite (buffer, sizeof (buffer), 1 , f);
    }
  fclose (f);
/*
  std::ofstream osf4("/tmp/ccnd.conf", std::fstream::trunc);
  osf4 << "add ccnx:/ " <<  ccnTransport << " 10.1.1.2 9695" << std::endl;
  osf4.close ();
  */
}

bool CompareNodeId (const ProcStatus& a, const ProcStatus& b)
{
   return (a.GetNode () < b.GetNode ());
}

int main (int argc, char *argv[])
{
  std::string packetSize = "1024";
  std::string ccnTransport = "udp"; // or tcp
#ifdef KERNEL_STACK
  bool useKernel = 0;
#endif
  bool useFork = 1;
  bool ccnTest = 0;
  int nbClients = 1;
  int taille = 6 * 1024;
  std::string outputFileName;
  std::string summaryFileName;

  CommandLine cmd;
  bool enableFlowMonitor = false;
  cmd.AddValue ("EnableMonitor", "Enable Flow Monitor", enableFlowMonitor);
#ifdef KERNEL_STACK
  cmd.AddValue ("kernel", "Use kernel linux IP stack.", useKernel);
#endif
  cmd.AddValue ("ccnd", "if true use ccnd to exchange the file, else use ttcp.", ccnTest);
  cmd.AddValue ("size", "packet size for ccnsendchunks. default 1024.", packetSize);
  cmd.AddValue ("ct", "CCN used transport udp or tcp. default udp.", ccnTransport);
  cmd.AddValue ("nb","Number of clients. default 1.", nbClients);
  cmd.AddValue ("total", "Downloaded file size in Ko default 6144", taille);
  cmd.AddValue ("f", "Use Fork (default YES).", useFork);

  cmd.Parse (argc, argv);

  std::ostringstream oss;

  oss << "Exp2-" << (ccnTest?"Ccn":"Tcp") << "-Node-Number-" << nbClients << "-Size-Transfert-" << taille << "-Kb" ;
  outputFileName = oss.str ();

  std::ostringstream oss2;
  oss2 << "Exp2-" << (ccnTest?"Ccn":"Tcp") << "-" << taille << "-Kb";

  summaryFileName = oss2.str ();

  NodeContainer nodes;
  nodes.Create (2);

  CreateFiles (ccnTransport,taille);

  NodeContainer nclients;
  nclients.Create (nbClients);
  DceManagerHelper dceManager;
#ifdef KERNEL_STACK
  if (!useKernel)
    {
#endif
      InternetStackHelper stack;
      stack.Install (nodes);
      stack.Install (nclients);
#ifdef KERNEL_STACK
    }
  else
    {
      dceManager.SetNetworkStack ("ns3::LinuxSocketFdFactory", "Library", StringValue ("liblinux.so"));
      LinuxStackHelper stack;
      stack.Install (nodes);
      stack.Install (nclients);
    }
#endif
  PointToPointHelper pointToPoint;
  pointToPoint.SetDeviceAttribute ("DataRate", StringValue ("10Mbps"));
  pointToPoint.SetChannelAttribute ("Delay", StringValue ("10ms"));

  Ipv4AddressHelper address;
  NetDeviceContainer devices;
  devices = pointToPoint.Install (nodes);

  address.SetBase ("10.1.1.0", "255.255.255.0");
  Ipv4InterfaceContainer p2pInterfaces;
  p2pInterfaces = address.Assign (devices);

  CsmaHelper csma;
  csma.SetChannelAttribute ("DataRate", StringValue ("1Gbps"));
  csma.SetChannelAttribute ("Delay", StringValue ("1ms"));

  NodeContainer csmaNodes;
  csmaNodes.Add (nodes.Get (1));
  csmaNodes.Add (nclients);

  NetDeviceContainer csmaDevices;
  csmaDevices = csma.Install (csmaNodes);

  address.SetBase ("10.1.2.0", "255.255.255.0");
  Ipv4InterfaceContainer csmaInterfaces;
  csmaInterfaces = address.Assign (csmaDevices);

  /*

  PointToPointHelper ppp;
  ppp.SetDeviceAttribute ("DataRate", StringValue ("10Mbps"));
  ppp.SetChannelAttribute ("Delay", StringValue ("1ms"));
   */

  Ipv4GlobalRoutingHelper::PopulateRoutingTables ();

  dceManager.Install (nodes);
  dceManager.Install (nclients);

  Names::Add ("NODE_Zero", nodes.Get (0));
  Names::Add ("NODE_One", nodes.Get (1));
  Names::Add ("NODE_Two", nclients.Get (0));

  //  csma.EnablePcap( "prefix",  csmaDevices.Get (0));
  //  csma.EnablePcap( "prefix",  csmaDevices.Get (1));

  CcnClientHelper dce;
  ApplicationContainer apps, putter, getter;

  dce.SetStackSize (1<<20);

  if (ccnTest)
    {
      // Launch ccnd on  NODE 0
      dce.SetBinary ("ccnd");
      dce.ResetArguments();
      dce.ResetEnvironment();
      dce.AddEnvironment("CCND_CAP", "50000");
      dce.AddEnvironment("CCN_LOCAL_PORT", "9695");
      dce.AddEnvironment("CCND_CAP", "");
      dce.AddEnvironment("CCND_AUTOREG", "");
      dce.AddEnvironment("CCND_LISTEN_ON", "");
      dce.AddEnvironment("CCND_MTU", "");
      dce.AddEnvironment("CCND_LOCAL_SOCKNAME", "");
      dce.AddEnvironment("CCND_DATA_PAUSE_MICROSEC", "");
      dce.AddEnvironment("CCND_KEYSTORE_DIRECTORY", "");
      apps = dce.Install (nodes.Get (0));
      apps.Start (Seconds (0.1));
      apps = dce.Install (nodes.Get (1));
      apps.Start (Seconds (0.1));

      for (int c=0; c < nbClients; c++)
        {
          if (!c)
            {
              dce.AddEnvironment ("CCND_DEBUG", "-1");
            }
          apps = dce.Install (nclients.Get (c));
          apps.Start (Seconds (0.1));
        }

      // Configure ccn daemon on node 1 to forward interrests to node 0
      dce.SetBinary ("ccndc");
      dce.ResetArguments();
      dce.ResetEnvironment();
      dce.AddEnvironment("HOME", "/root");
      dce.AddArgument ("-v");
      dce.AddArgument ("add");
      dce.AddArgument ("/");
      dce.AddArgument (ccnTransport);
      dce.AddArgument ("10.1.1.1");
      apps = dce.Install (nodes.Get (1));
      apps.Start (Seconds (0.2));

      // PUBLISH DATA FILE ON NODE 0
      dce.ResetArguments();
      dce.ResetEnvironment();
      dce.SetBinary ("ccnsendchunks");
      dce.AddEnvironment("HOME", "/root");
      dce.SetStdinFile ("/tmp/bigone");
      dce.AddFile ("/tmp/bigone", "/tmp/bigone");
      dce.AddArgument ("-b");
      dce.AddArgument (packetSize);
      dce.AddArgument ("-x");
      dce.AddArgument ("2000"); // 2147  is the max as visible in great ccnx source code !
      dce.AddArgument ("ccnx:/DATAFILE");
      putter = dce.Install (nodes.Get (0));
      putter.Start (Seconds (0.3));


      for (int c=0; c < nbClients; c++)
        {
          dce.SetBinary ("ccndc");
          dce.ResetArguments();
          dce.ResetEnvironment();
          dce.AddEnvironment("HOME", "/root");
          dce.AddArgument ("-v");
          dce.AddArgument ("add");
          dce.AddArgument ("/");
          dce.AddArgument (ccnTransport);
          dce.AddArgument ("10.1.2.1");
          apps = dce.Install (nclients.Get (c));
          apps.Start (Seconds (0.1));

          // NODE 2 + C RETRIEVE THE DATA FILE
          dce.ResetArguments();
          dce.ResetEnvironment();
          if (useFork)
            {
              dce.SetBinary ("lanceur");
            }
          else
            {
              dce.SetBinary ("ccncatchunks2");
              dce.AddArgument ("-d");
              dce.AddArgument ("-a");
              dce.AddArgument ("ccnx:/DATAFILE");
            }
          getter = dce.Install (nclients.Get (c));
          getter.Start (Seconds (1.0));
        }

      // STOP CCND ON EACH NODES
      dce.ResetArguments();
      dce.ResetEnvironment();
      dce.SetBinary ("ccndsmoketest");
      dce.SetStdinFile ("");
      dce.AddArgument ("kill");
      apps = dce.Install (nodes.Get (0));
      apps.Start (Seconds (1990.0));
      apps = dce.Install (nodes.Get (1));
      apps.Start (Seconds (1990.0));

      for (int c=0; c < nbClients; c++)
        {
          apps = dce.Install (nclients.Get (c));
          apps.Start (Seconds (1990.0));
        }

      //pointToPoint.EnablePcapAll ( std::string("dce-ccn-xchg-")+packetSize, false);
    }
  else
    {
      std::ostringstream oss;

      oss << (taille * 1024);

      dce.ResetArguments();
      dce.ResetEnvironment();
      dce.SetBinary ("my-serveur");
      dce.AddArgument ("5000");
      dce.AddArgument (oss.str()); // size
      oss.clear ();
      oss.str ("");
      oss << nbClients;
      dce.AddArgument (oss.str()); // Stop after nbClients

      apps = dce.Install (nodes.Get (0));
      // apps = dce.Install (nodes.Get (1));
      apps.Start (Seconds (0.3));

      for (int c=0; c < nbClients; c++)
        {
          dce.ResetArguments();
          dce.ResetEnvironment();
          dce.SetBinary ("my-client");
          dce.AddArgument ("10.1.1.1");
          //dce.AddArgument ("10.1.2.1");
          dce.AddArgument ("5000");
          oss.clear ();
          oss.str ("");
          oss << (taille * 1024);
          dce.AddArgument (oss.str());

          apps = dce.Install (nclients.Get (c));
          //  apps = dce.Install (nodes.Get (1));
          apps.Start (Seconds (1.0));
        }
      //pointToPoint.EnablePcapAll ("dce-ttcp-xchg", false);
    }

  //  csma.EnablePcap( "prefix",  csmaDevices.Get (1));

  // Flow Monitor
  Ptr<FlowMonitor> flowmon;
  if (enableFlowMonitor)
    {
      FlowMonitorHelper flowmonHelper;
      flowmon = flowmonHelper.InstallAll ();
    }

  GlobalValue::Bind ("ChecksumEnabled", BooleanValue (true));

  if (ccnTest)
    {
      Simulator::Stop (Seconds(2001.0));
    }
  else
    {
      Simulator::Stop (Seconds(3600.0*24));
    }
  std::remove("exitprocs");

  Simulator::Run ();

  if (enableFlowMonitor)
    {
      flowmon->SerializeToXmlFile (outputFileName+".flowmon", false, false);
    }

  //  Simulator::Destroy ();

  std::string sumFn = summaryFileName + "-Max-Time.txt";
  std::string sumFn2 = summaryFileName + "-Average-Time.txt";

  if (ccnTest)
    {
      int nSuccess = 0;
      long maxTime = 0;
      int nFailures = 0;
      std::string fn = outputFileName + ".time-per-node";
      std::ofstream timepernode( fn.c_str (), std::fstream::trunc);
      long totalTime = 0;

      if (!useFork)
        {
          std::vector <ProcStatus> v = dceManager.GetProcStatus ();

          /*
          struct sort_by_nodeid
          {
            inline bool operator() (const ProcStatus& a, const ProcStatus& b) const
                {
                    return (a.GetNode () < b.GetNode ());
                }
          };
          */
          std::sort (v.begin (), v.end (), CompareNodeId );

          std::cout << "Nb Process " << v.size () << std::endl;

          for (std::vector <ProcStatus>::iterator i = v.begin (); i != v.end () ; ++i)
            {
              ProcStatus st = *i;
              const char *interest = "ccncatchunks2";
              if (st.GetCmdLine().compare(0, strlen (interest), interest) == 0)
                {
                  std::cout << "Duration: " << st.GetSimulatedDuration () << std::endl;

                  if (!st.GetExitCode())
                    {
                      nSuccess++;
                      long duration = (long) ( 1000L *   st.GetSimulatedDuration () );;

                      timepernode << "Node " << st.GetNode() << ' ' << duration << std::endl;

                      totalTime += duration;
                      if (duration > maxTime)
                        {
                          maxTime = duration;
                        }
                    }
                  else
                    {
                      nFailures++;
                    }
                }
            }
        }
      else
        {
          for (int c=0; c < nbClients; c++)
            {
              int nodeId = c + 2;
              char filename[1024];
              struct stat st;

              sprintf (filename,"files-%d/var/log/SUCCESS.TXT", nodeId);

              if ( 0 == stat (filename, &st) )
                {
                  nSuccess++;
                  FILE *f = fopen (filename ,"r");
                  long duration = 0;
                  if (fscanf (f, "%ld", &duration) > 0)
                    {
                      timepernode << "Node " << nodeId << ' ' << duration << std::endl;

                      totalTime += duration;
                      if (duration > maxTime)
                        {
                          maxTime = duration;
                        }
                    }
                  fclose (f);
                }
              else
                {
                  sprintf (filename,"files-%d/var/log/FAILURE.TXT", nodeId);

                  if ( 0 == stat (filename, &st) )
                    {
                      nFailures++;
                    }
                }
            }
        }
      timepernode.close ();
      printf ("Synthese %d OK, %d KO, total duration %ld ms.\n", nSuccess, nFailures, maxTime);

      FILE *f = fopen (sumFn.c_str (), "a+");
      fprintf (f, "%d  %ld # %s\n", nSuccess ,maxTime,outputFileName.c_str ());
      fclose (f);

      if (0 != nSuccess)
        {
          f = fopen (sumFn2.c_str (), "a+");
          fprintf (f, "%d  %ld # %s\n", nSuccess ,totalTime / nSuccess ,outputFileName.c_str ());
          fclose (f);
        }
    }
  else
    {
      char *filename = "files-0/var/log/duration.txt";
      struct stat st;
      std::string fn;


      if ( 0 == stat (filename, &st) )
        {
          FILE *f = fopen (filename ,"r");
          long duration = 0;
          if (fscanf (f, "%ld", &duration) > 0)
            {
              FILE *ff = fopen (sumFn.c_str (), "a+");
              fprintf (ff, "%d  %ld # %s\n", nbClients, duration, outputFileName.c_str ());
              fclose (ff);
            }
        }
    }

  return 0;
}


